/*!
 * LockingColumnHeaderGroup v1.8 for Ext 3.1 and 3.2
 */
Ext.namespace("Ext.ux.grid");
Ext.ux.grid.LockingColumnHeaderGroup = Ext.extend(Object, {
    constructor: function (config) {
        this.config = config;
        var rows = config.rows,
            defaults = config.defaults,
            row, rlen, r, i, gcol, len;
        if (defaults) {
            for (row = 0, rlen = rows.length; row < rlen; row++) {
                r = rows[row];
                for (i = 0, gcol = 0, len = r.length; i < len; i++) {
                    Ext.applyIf(r[i], defaults);
                }
            }
        }
    },
    init: function (grid) {
        grid.enableColLock = false;
        Ext.applyIf(grid.colModel, this.config);
        Ext.apply(grid.getView(), this.viewConfig);
    },
    viewConfig: {
        initTemplates: function () {
            this.constructor.prototype.initTemplates.apply(this, arguments);
            var ts = this.templates || {};
            if (!ts.gcell) {
                ts.gcell = new Ext.XTemplate('<td class="x-grid3-hd x-grid3-gcell x-grid3-td-{id} ux-grid-hd-group-row-{row} {cls}" style="{style}">',
                    '<div {tooltip} class="x-grid3-hd-inner x-grid3-hd-{id}" unselectable="on" style="{istyle}">',
                    this.grid.enableHdMenu ? '<a class="x-grid3-hd-btn" href="#"></a>' : '',
                    '{value}</div></td>');
            }
            this.templates = ts;
            this.hrowRe = new RegExp("ux-grid-hd-group-row-(\\d+)", "");
        },
        renderHeaders: function () {
            var ts = this.templates,
                headers = [[], []],
                cells,
                cm = this.cm,
                rows = cm.rows,
                tstyle = ['width:' + this.getTotalWidth() + ';', 'width:' + this.getLockedWidth() + ';'];
            for (var row = 0, rlen = rows.length; row < rlen; row++) {
                var r = rows[row];
                cells = [[], []];
                for (var i = 0, gcol = 0, len = r.length; i < len; i++) {
                    var group = r[i];
                    group.colspan = group.colspan || 1;
                    var ix = group.dataIndex ? cm.findColumnIndex(group.dataIndex) : gcol,
                        id = this.getColumnId(ix),
                        gs = Ext.ux.grid.LockingColumnHeaderGroup.prototype.getGroupStyle.call(this, group, gcol);
                    cells[cm.isLocked(ix) ? 1 : 0][i] = ts.gcell.apply({
                        cls: 'ux-grid-hd-group' +
                            (group.header && !group.partialHeader ? ' ux-grid-hd-group-cell' : '') +
                            (group.groupCls ? ' ' + group.groupCls : ''),
                        id: id,
                        row: row,
                        style: 'width:' + gs.width + ';' + (gs.hidden ? 'display:none;' : '') + (group.align ? 'text-align:' + group.align + ';' : '') + (group.groupStyle ? group.groupStyle : ''),
                        tooltip: group.tooltip ? (Ext.QuickTips.isEnabled() ? 'ext:qtip' : 'title') + '="' + group.tooltip + '"' : '',
                        istyle: group.align == 'right' ? 'padding-right:16px' : '',
                        btn: this.grid.enableHdMenu && group.header,
                        value: group.header || '&nbsp;'
                    });
                    gcol += group.colspan;
                }
                headers[0][row] = ts.header.apply({
                    tstyle: tstyle[0],
                    cells: cells[0].join('')
                });
                headers[1][row] = ts.header.apply({
                    tstyle: tstyle[1],
                    cells: cells[1].join('')
                });
            }
            cells = this.constructor.prototype.renderHeaders.apply(this, arguments);
            headers[0].push(cells[0]);
            headers[1].push(cells[1]);
            return [headers[0].join(''), headers[1].join('')];
        },
        onColumnWidthUpdated: function () {
            this.constructor.prototype.onColumnWidthUpdated.apply(this, arguments);
            Ext.ux.grid.LockingColumnHeaderGroup.prototype.updateGroupStyles.call(this);
        },
        onAllColumnWidthsUpdated: function () {
            this.constructor.prototype.onAllColumnWidthsUpdated.apply(this, arguments);
            Ext.ux.grid.LockingColumnHeaderGroup.prototype.updateGroupStyles.call(this);
        },
        onColumnHiddenUpdated: function () {
            this.constructor.prototype.onColumnHiddenUpdated.apply(this, arguments);
            Ext.ux.grid.LockingColumnHeaderGroup.prototype.updateGroupStyles.call(this);
        },
        getHeaderCell: function (index) {
            var llen = this.cm.getLockedCount();
            if (index < llen) {
                return this.lockedHd.query(this.cellSelector)[index];
            }
            return this.mainHd.query(this.cellSelector)[index - llen];
        },
        findHeaderCell: function (el) {
            return el ? this.fly(el).findParent('td.x-grid3-hd', this.cellSelectorDepth) : false;
        },
        findHeaderIndex: function (el) {
            var cell = this.findHeaderCell(el);
            return cell ? this.getCellIndex(cell) : false;
        },
        updateSortIcon: function (col, dir) {
            var sc = this.sortClasses,
                lhds = this.lockedHd.select(this.cellSelector).removeClass(sc),
                hds = this.mainHd.select(this.cellSelector).removeClass(sc),
                llen = this.cm.getLockedCount();
            if (col < llen) {
                lhds.item(col).addClass(sc[dir == 'DESC' ? 1 : 0]);
            } else {
                hds.item(col - llen).addClass(sc[dir == 'DESC' ? 1 : 0]);
            }
        },
        handleHdOver : function(e, t){
            var hd = this.findHeaderCell(t);
            if(hd && !this.headersDisabled){
                this.activeHdRef = t;
                this.activeHdIndex = this.getCellIndex(hd);
                var fly = this.fly(hd);
                this.activeHdRegion = fly.getRegion();
                var row = Ext.ux.grid.LockingColumnHeaderGroup.prototype.getGroupRowIndex.call(this, hd),
                    group = Ext.ux.grid.LockingColumnHeaderGroup.prototype.getGroupSpan.call(this, row, this.activeHdIndex).group;
                if(group ? !group.menuDisabled : !this.cm.isMenuDisabled(this.activeHdIndex)){
                    fly.addClass('x-grid3-hd-over');
                    this.activeHdBtn = fly.child('.x-grid3-hd-btn');
                    if(this.activeHdBtn){
                        this.activeHdBtn.dom.style.height = (hd.firstChild.offsetHeight-1)+'px';
                    }
                }
            }
        },
        handleHdMove : function(e, t){
            var hd = this.findHeaderCell(this.activeHdRef);
            if(hd && !this.headersDisabled){
                var hw = this.splitHandleWidth || 5,
                    r = this.activeHdRegion,
                    x = e.getPageX(),
                    ss = hd.style,
                    cur = '';
                if(this.grid.enableColumnResize !== false){
                    var row = Ext.ux.grid.LockingColumnHeaderGroup.prototype.getGroupRowIndex.call(this, hd),
                        group1 = this.activeHdIndex > 0 ? Ext.ux.grid.LockingColumnHeaderGroup.prototype.getGroupSpan.call(this, row, this.activeHdIndex - 1).group : null,
                        group2 = Ext.ux.grid.LockingColumnHeaderGroup.prototype.getGroupSpan.call(this, row, this.activeHdIndex).group;
                    if(x - r.left <= hw && (group1 ? group1.resizable !== false : this.cm.isResizable(this.activeHdIndex-1))){
                        cur = Ext.isAir ? 'move' : Ext.isWebKit ? 'e-resize' : 'col-resize'; // col-resize not always supported
                    }else if(r.right - x <= (!this.activeHdBtn ? hw : 2) && (group2 ? group2.resizable !== false : this.cm.isResizable(this.activeHdIndex))){
                        cur = Ext.isAir ? 'move' : Ext.isWebKit ? 'w-resize' : 'col-resize';
                    }
                }
                ss.cursor = cur;
            }
        },
        handleHdMenuClick: function (item) {
            var index = this.hdCtxIndex,
                cm = this.cm,
                ds = this.ds,
                id = item.getItemId();
            switch (id) {
            case 'asc':
                ds.sort(cm.getDataIndex(index), 'ASC');
                break;
            case 'desc':
                ds.sort(cm.getDataIndex(index), 'DESC');
                break;
            default:
                if (id.substr(0, 5) == 'group') {
                    var sp = id.split('-'),
                        row = parseInt(sp[1], 10),
                        col = parseInt(sp[2], 10),
                        r = this.cm.rows[row],
                        group, gcol = 0,
                        i, len;
                    for (i = 0, len = r.length; i < len; i++) {
                        group = r[i];
                        if (col >= gcol && col < gcol + group.colspan) {
                            break;
                        }
                        gcol += group.colspan;
                    }
                    if (item.checked) {
                        var max = cm.getColumnsBy(this.isHideableColumn, this).length;
                        for (i = gcol, len = gcol + group.colspan; i < len; i++) {
                            if (!cm.isHidden(i)) {
                                max--;
                            }
                        }
                        if (max < 1) {
                            this.onDenyColumnHide();
                            return false;
                        }
                    }
                    for (i = gcol, len = gcol + group.colspan; i < len; i++) {
                        if (cm.config[i].fixed !== true && cm.config[i].hideable !== false) {
                            cm.setHidden(i, item.checked);
                        }
                    }
                } else {
                    index = cm.getIndexById(id.substr(4));
                    if (index != -1) {
                        if (item.checked && cm.getColumnsBy(this.isHideableColumn, this).length <= 1) {
                            this.onDenyColumnHide();
                            return false;
                        }
                        cm.setHidden(index, item.checked);
                    }
                }
                item.checked = !item.checked;
                if (item.menu) {
                    var updateChildren = function (menu) {
                        menu.items.each(function (childItem) {
                            if (!childItem.disabled) {
                                childItem.setChecked(item.checked, false);
                                if (childItem.menu) {
                                    updateChildren(childItem.menu);
                                }
                            }
                        });
                    };
                    updateChildren(item.menu);
                }
                var parentMenu = item,
                    parentItem, isChecked = function (m) {
                        return m.checked;
                    };
                while ((parentMenu = parentMenu.parentMenu)) {
                    if (!parentMenu.parentMenu || !(parentItem = parentMenu.parentMenu.items.get(parentMenu.getItemId())) || !parentItem.setChecked) {
                        break;
                    }
                    var checked = parentMenu.items.findIndexBy(isChecked) >= 0;
                    parentItem.setChecked(checked, true);
                }
                item.checked = !item.checked;
            }
            return true;
        },
        beforeColMenuShow: function () {
            var cm = this.cm,
                rows = this.cm.rows;
            this.colMenu.removeAll();
            for (var col = 0, clen = cm.getColumnCount(); col < clen; col++) {
                var menu = this.colMenu,
                    title = cm.getColumnHeader(col),
                    text = [];
                if (cm.config[col].fixed !== true && cm.config[col].hideable !== false) {
                    for (var row = 0, rlen = rows.length; row < rlen; row++) {
                        var r = rows[row],
                            group, gcol = 0;
                        for (var i = 0, len = r.length; i < len; i++) {
                            group = r[i];
                            if (col >= gcol && col < gcol + group.colspan) {
                                break;
                            }
                            gcol += group.colspan;
                        }
                        if (group && group.header) {
                            if (cm.hierarchicalColMenu) {
                                var gid = 'group-' + row + '-' + gcol,
                                    item = menu.items.item(gid),
                                    submenu = item ? item.menu : null;
                                if (!submenu) {
                                    submenu = new Ext.menu.Menu({
                                        itemId: gid
                                    });
                                    submenu.on("itemclick", this.handleHdMenuClick, this);
                                    var checked = false,
                                        disabled = true;
                                    for (var c = gcol, lc = gcol + group.colspan; c < lc; c++) {
                                        if (!cm.isHidden(c)) {
                                            checked = true;
                                        }
                                        if (cm.config[c].hideable !== false) {
                                            disabled = false;
                                        }
                                    }
                                    menu.add({
                                        itemId: gid,
                                        text: group.header,
                                        menu: submenu,
                                        hideOnClick: false,
                                        checked: checked,
                                        disabled: disabled
                                    });
                                }
                                menu = submenu;
                            } else {
                                text.push(group.header);
                            }
                        }
                    }
                    text.push(title);
                    menu.add(new Ext.menu.CheckItem({
                        itemId: "col-" + cm.getColumnId(col),
                        text: text.join(' '),
                        checked: !cm.isHidden(col),
                        hideOnClick: false,
                        disabled: cm.config[col].hideable === false
                    }));
                }
            }
        },
        renderUI: function () {
            this.constructor.prototype.renderUI.apply(this, arguments);
            Ext.apply(this.columnDrop, Ext.ux.grid.LockingColumnHeaderGroup.prototype.columnDropConfig);
        },
        syncHeaderHeight: function () {
        	var hds = this.innerHd.firstChild.childNodes,
        	    lhds = this.lockedInnerHd.firstChild.childNodes;
            for (var i = 0, len = hds.length; i < len; i++) {
                hds[i].firstChild.firstChild.style.height = 'auto';
                lhds[i].firstChild.firstChild.style.height = 'auto';
                var hd = hds[i].firstChild.firstChild.offsetHeight,
                    lhd = lhds[i].firstChild.firstChild.offsetHeight,
                    height = (lhd > hd ? lhd : hd) + 'px';
                hds[i].firstChild.firstChild.style.height = height;
                lhds[i].firstChild.firstChild.style.height = height;
            }
        },
        updateLockedWidth: function () {
            var lw = this.cm.getTotalLockedWidth(),
                tw = this.cm.getTotalWidth() - lw,
                csize = this.grid.getGridEl().getSize(true),
                lp = Ext.isBorderBox ? 0 : this.lockedBorderWidth,
                rp = Ext.isBorderBox ? 0 : this.rowBorderWidth,
                vw = (csize.width - lw - lp - rp) + 'px',
                so = this.getScrollOffset();
            if (!this.grid.autoHeight) {
                var vh = (csize.height - this.mainHd.getHeight()) + 'px';
                this.lockedScroller.dom.style.height = vh;
                this.scroller.dom.style.height = vh;
            }
            this.lockedWrap.dom.style.width = (lw + rp) + 'px';
            this.scroller.dom.style.width = vw;
            this.mainWrap.dom.style.left = (lw + lp + rp) + 'px';
            if (this.innerHd) {
                this.lockedInnerHd.firstChild.style.width = lw + 'px';
                this.innerHd.style.width = vw;
                this.innerHd.firstChild.style.width = (tw + rp + so) + 'px';
                for (var i = 0, len = this.innerHd.firstChild.childNodes.length; i < len; i++) {
                    this.lockedInnerHd.firstChild.childNodes[i].style.width = lw + 'px';
                    this.innerHd.firstChild.childNodes[i].style.width = tw + 'px';
                }
            }
            if (this.mainBody) {
                this.lockedBody.dom.style.width = (lw + rp) + 'px';
                this.mainBody.dom.style.width = (tw + rp) + 'px';
            }
        }
    },
    columnDropConfig: {
        getTargetFromEvent: function (e) {
            var t = Ext.lib.Event.getTarget(e);
            return this.view.findHeaderCell(t);
        },
        positionIndicator: function (h, n, e) {
            var data = Ext.ux.grid.LockingColumnHeaderGroup.prototype.getDragDropData.call(this, h, n, e);
            if (data === false) {
                return false;
            }
            var px = data.px + this.proxyOffsets[0];
            this.proxyTop.setLeftTop(px, data.r.top + this.proxyOffsets[1]);
            this.proxyTop.show();
            this.proxyBottom.setLeftTop(px, data.r.bottom);
            this.proxyBottom.show();
            return data.pt;
        },
        onNodeDrop: function (n, dd, e, data) {
            var h = data.header;
            if (h != n) {
                var d = Ext.ux.grid.LockingColumnHeaderGroup.prototype.getDragDropData.call(this, h, n, e);
                if (d === false) {
                    return false;
                }
                var cm = this.grid.colModel,
                    right = d.oldIndex < d.newIndex,
                    rows = cm.rows;
                for (var row = d.row, rlen = rows.length; row < rlen; row++) {
                    var r = rows[row],
                        len = r.length,
                        fromIx = 0,
                        span = 1,
                        toIx = len;
                    for (var i = 0, gcol = 0; i < len; i++) {
                        var group = r[i];
                        if (d.oldIndex >= gcol && d.oldIndex < gcol + group.colspan) {
                            fromIx = i;
                        }
                        if (d.oldIndex + d.colspan - 1 >= gcol && d.oldIndex + d.colspan - 1 < gcol + group.colspan) {
                            span = i - fromIx + 1;
                        }
                        if (d.newIndex >= gcol && d.newIndex < gcol + group.colspan) {
                            toIx = i;
                        }
                        gcol += group.colspan;
                    }
                    var groups = r.splice(fromIx, span);
                    rows[row] = r.splice(0, toIx - (right ? span : 0)).concat(groups).concat(r);
                }
                for (var c = 0; c < d.colspan; c++) {
                    var oldIx = d.oldIndex + (right ? 0 : c),
                        newIx = d.newIndex + (right ? -1 : c);
                    cm.moveColumn(oldIx, newIx);
                    this.grid.fireEvent("columnmove", oldIx, newIx);
                }
                return true;
            }
            return false;
        }
    },
    getGroupStyle: function (group, gcol) {
        var width = 0,
            hidden = true;
        for (var i = gcol, len = gcol + group.colspan; i < len; i++) {
            if (!this.cm.isHidden(i)) {
                var cw = this.cm.getColumnWidth(i);
                if (typeof cw == 'number') {
                    width += cw;
                }
                hidden = false;
            }
        }
        return {
            width: (Ext.isBorderBox || (Ext.isWebKit && !Ext.isSafari2) ? width : Math.max(width - this.borderWidth, 0)) + 'px',
            hidden: hidden
        };
    },
    updateGroupStyles: function (col) {
        var tables = [this.mainHd.query('.x-grid3-header-offset > table'), this.lockedHd.query('.x-grid3-header-offset > table')],
            tw = [this.getTotalWidth(), this.getLockedWidth()],
            rows = this.cm.rows;
        for (var row = 0; row < tables.length; row++) {
            tables[0][row].style.width = tw[0];
            tables[1][row].style.width = tw[1];
            if (row < rows.length) {
                var cells = [tables[0][row].firstChild.firstChild.childNodes, tables[1][row].firstChild.firstChild.childNodes];
                var llen = cells[1].length,
                    tlen = llen + cells[0].length;
                for (var i = 0, gcol = 0; i < tlen; i++) {
                    var group = rows[row][i];
                    if ((typeof col != 'number') || (col >= gcol && col < gcol + group.colspan)) {
                        var gs = Ext.ux.grid.LockingColumnHeaderGroup.prototype.getGroupStyle.call(this, group, gcol);
                        var cell = i < llen ? cells[1][i] : cells[0][i - llen];
                        cell.style.width = gs.width;
                        cell.style.display = gs.hidden ? 'none' : '';
                    }
                    gcol += group.colspan;
                }
            }
        }
    },
    getGroupRowIndex: function (el) {
        if (el) {
            var m = el.className.match(this.hrowRe);
            if (m && m[1]) {
                return parseInt(m[1], 10);
            }
        }
        return this.cm.rows.length;
    },
    getGroupSpan: function (row, col) {
        if (row < 0) {
            return {
                col: 0,
                colspan: this.cm.getColumnCount()
            };
        }
        var r = this.cm.rows[row];
        if (r) {
            for (var i = 0, gcol = 0, len = r.length; i < len; i++) {
                var group = r[i];
                if (col >= gcol && col < gcol + group.colspan) {
                    return {
                        col: gcol,
                        colspan: group.colspan,
                        group: group
                    };
                }
                gcol += group.colspan;
            }
            return {
                col: gcol,
                colspan: 0
            };
        }
        return {
            col: col,
            colspan: 1
        };
    },
    getDragDropData: function (h, n, e) {
        if (h.parentNode != n.parentNode) {
            return false;
        }
        var cm = this.grid.colModel,
            x = Ext.lib.Event.getPageX(e),
            r = Ext.lib.Dom.getRegion(n.firstChild),
            px, pt;
        if ((r.right - x) <= (r.right - r.left) / 2) {
            px = r.right + this.view.borderWidth;
            pt = "after";
        } else {
            px = r.left;
            pt = "before";
        }
        var oldIndex = this.view.getCellIndex(h),
            newIndex = this.view.getCellIndex(n);
        if (cm.isFixed(newIndex)) {
            return false;
        }
        var row = Ext.ux.grid.LockingColumnHeaderGroup.prototype.getGroupRowIndex.call(this.view, h),
            oldGroup = Ext.ux.grid.LockingColumnHeaderGroup.prototype.getGroupSpan.call(this.view, row, oldIndex),
            newGroup = Ext.ux.grid.LockingColumnHeaderGroup.prototype.getGroupSpan.call(this.view, row, newIndex);
        oldIndex = oldGroup.col;
        newIndex = newGroup.col + (pt == "after" ? newGroup.colspan : 0);
        if (newIndex >= oldGroup.col && newIndex <= oldGroup.col + oldGroup.colspan) {
            return false;
        }
        var parentGroup = Ext.ux.grid.LockingColumnHeaderGroup.prototype.getGroupSpan.call(this.view, row - 1, oldIndex);
        if (newIndex < parentGroup.col || newIndex > parentGroup.col + parentGroup.colspan) {
            return false;
        }
        return {
            r: r,
            px: px,
            pt: pt,
            row: row,
            oldIndex: oldIndex,
            newIndex: newIndex,
            colspan: oldGroup.colspan
        };
    }
});
