'use strict';
if (typeof console == "undefined" || typeof console.log == "undefined")
    var console = {
        log : function() {
        }
    };

// namespace definitions
Ext.ns('WISA');
Ext.ns('WISA.Forms');
Ext.ns('WISA.controls');
Ext.ns('WISA.data');
Ext.ns('WISA.baseforms');

// /* Error Handling */
WISA.showError = function(title, message) {
    Ext.Msg.show({
        title : title,
        msg : message,
        icon : Ext.Msg.ERROR,
        buttons : Ext.Msg.OK
    });
};
WISA.showAjaxError = function(message) {
    WISA.showError('Ajax Error', message);
};
WISA.getResponseError = function(provider, response, message) {
    var result = (!Ext.isEmpty(response)) && (Ext.isEmpty(response.error)) && (Ext.isDefined(response.result));
    if (!result) {
        if ((!Ext.isEmpty(response)) && (!Ext.isEmpty(response.error))) {
            message = message + response.error.message;
        } else {
            message = message + 'Interne server fout.';
        }
        WISA.showAjaxError(message);
    }
    return result;
};

// /* Server Configuration */
WISA.getServerConfiguration = function(options) {
    var urlProvider = Application.getUrlApplication() + '/Provider/' + options.ProviderName + '/';
    var objHttpProxy = new Ext.data.HttpProxy({
        api : {
            read : urlProvider + 'Read/',
            update : urlProvider + 'Update/',
            create : urlProvider + 'Insert/',
            destroy : urlProvider + 'Delete/'
        }
    });
    // We expect metadata from the server.
    var objJsonReader = new Ext.data.JsonReader({
        root : 'rows',
        messageProperty : 'message',
        successProperty : 'success'
    });
    var objJsonWriter = new Ext.data.JsonWriter({
        encode : true,
        writeAllFields : true
    });
    return {
        reader : objJsonReader,
        writer : objJsonWriter,
        proxy : objHttpProxy
    };
};

// /* Basic Components */

/**
 * @class WISA.controls.Label
 * @information The basic control to show non editable text on the form.
 * @extends Ext.form.Label
 */
WISA.controls.Label = Ext.extend(Ext.form.Label, {});

/**
 * @class WISA.controls.TextField
 * @information The basic edit control for all WISA applications.
 * @extends Ext.form.TextField
 */
WISA.controls.TextField = Ext.extend(Ext.form.TextField, {
  minLengthText: 'De waarde moet minstens {minLength} tekens bevatten',
  maxLengthText: 'De waarde mag niet meer dan {maxLength} tekens bevatten',
  blankText: 'Dit is een verplicht veld'
});

/**
 * @class WISA.controls.NumberField
 * @information The basic number control for all WISA applications.
 * @extends Ext.form.NumberField
 */
WISA.controls.NumberField = Ext.extend(Ext.form.NumberField, {});

/**
 * @class WISA.controls.DateField
 * @information The basic datum control for all WISA applications.
 * @extends Ext.form.DateField
 */
WISA.controls.DateField = Ext.extend(Ext.form.DateField, {});

/**
 * @class WISA.controls.TimeField
 * @information The basic time control for all WISA applications.
 * @extends Ext.form.TimeField
 */
WISA.controls.TimeField = Ext.extend(Ext.form.TimeField, {});

/**
 * @class WISA.controls.ComboBox
 * @information The basic combo box for all WISA applications.
 * @extends Ext.form.ComboBox
 */
WISA.controls.ComboBox = Ext.extend(Ext.form.ComboBox, {
  triggerAction:'all',
  lazyInit: false,
  forceSelection: true,
  mode: 'local',
  valueNotFoundText : 'Waarde niet gevonden',
});

/**
 * @class WISA.controls.CheckBox
 * @information The basic check box for all WISA applications.
 * @extends Ext.form.Checkbox
 */
WISA.controls.CheckBox = Ext.extend(Ext.form.Checkbox, {});

/**
 * @class WISA.controls.DateField
 * @information The basic datum control for all WISA applications.
 * @extends Ext.form.Radio
 */
WISA.controls.Radio = Ext.extend(Ext.form.Radio, {});

/**
 * @class WISA.controls.RadioGroup
 * @information The basic radio buttons for all WISA applications.
 * @extends Ext.form.RadioGroup
 */
WISA.controls.RadioGroup = Ext.extend(Ext.form.RadioGroup, {});

/**
 * @class WISA.controls.HtmlEditor
 * @information The basic HTML editor for all WISA applications.
 * @extends Ext.form.HtmlEditor
 */
WISA.controls.HtmlEditor = Ext.extend(Ext.form.HtmlEditor, {});

/**
 * @class WISA.controls.TextArea
 * @information The basic memo control for all WISA applications.
 * @extends Ext.form.TextArea
 */
WISA.controls.TextArea = Ext.extend(Ext.form.TextArea, {});

/**
 * @class WISA.controls.Panel
 * @information The basic Panel for all WISA applications.
 * @extends Ext.Panel
 */
WISA.controls.Panel = Ext.extend(Ext.Panel, {});

/**
 * @class WISA.controls.Grid
 * @information The basic grid for all WISA applications.
 * @extends Ext.grid.GridPanel
 * @author Michael Van Canneyt
 * @version 0.9
 */
WISA.controls.Grid = Ext.extend(Ext.grid.GridPanel, {});
/**
 * @class WISA.controls.LookupComboBox
 * @information Lookup combo box with built-in store.
 * @extends Ext.form.LookupComboBox
 * @author Michael Van Canneyt
 * @version 0.9
 */
WISA.controls.LookupComboBox = Ext.extend(WISA.controls.ComboBox,{
  constructor : function(options) {
     var mystore = new WISA.ClientDataset({
        ProviderName : options.ProviderName,
        autoLoad: Ext.isEmpty(options.autoLoad) ? true : options.autoLoad,
        listeners : {
          'load': { fn: this.storeLoaded, scope : this }
        }
      });
    var defaultID = null;
    if (Ext.isDefined(options.defaultID)) {
      defaultID=options.defaultID;
    };
    Ext.apply(options,{
      store : mystore,
      valueNotFoundText : '(Waarde niet gevonden)',
    });
    // Getter for DefaultID
    this.getDefaultID = function () {
      return defaultID;
    };
    // Setter for DefaultID
    this.setDefaultID = function (value) {
      defaultID=value;
      checkDefaultID();
    };
    // Superclass Constructor
    WISA.controls.LookupComboBox.superclass.constructor.call(this, options);
    // Triggered when schools have been loaded.
  },
  // Triggered when store has been loaded.
  storeLoaded : function (store,records, options) {
    if (records.length == 1) {
      this.setValue(records[0].get(this.initialConfig.valueField));
    } else {
      this.checkDefaultID();
    }
  },
  // Used to check if a default ID must/can be loaded
  checkDefaultID : function  () {
    var s = this.getStore();
    if (( !Ext.isEmpty(this.getDefaultID())) && (s.records.length>0)) {
      var i = s.findExact(this.initialConfig.valueField,this.getDefaultID());
      if (i != -1) {
        this.setValue(this.getDefaultID());
      }
    }
  }
});

/**
 * @class WISA.data.ClientDataset
 * @extends Ext.Datastore
 * @information The basic data store for all WISA applications. <br />
 *              Config is an object that must contain
 *              <ul>
 *              <li>ProviderName : String with name of provider.</li>
 *              <li>Fields: List of fieldnames to expect.</li>
 *              <li>IDField : String with name of ID field of data.</li>
 *              </ul>
 * @author Michael Van Canneyt
 * @version 0.9
 */
WISA.data.ClientDataset = Ext.extend(Ext.data.Store, {
    // Constructor
    constructor : function(options) {
        var optionsServer = WISA.getServerConfiguration(options);
        // Merge data
        Ext.apply(options, optionsServer);
        WISA.data.ClientDataset.superclass.constructor.call(this, options);
    }
});

/**
 * @class WISA.data.GroupingClientDataset
 * @extends Ext.Data.GroupingStore
 * @information The basic data store for all WISA applications. <br />
 *              Config is an object that must contain
 *              <ul>
 *              <li>ProviderName : String with name of provider.</li>
 *              <li>Fields: List of fieldnames to expect.</li>
 *              <li>IDField : String with name of ID field of data.</li>
 *              </ul>
 * 
 * @author Michael Van Canneyt
 * @version 0.9
 */
WISA.data.GroupingClientDataset = Ext.extend(Ext.data.GroupingStore, {
    // Constructor
    constructor : function(options) {
        var optionsServer = WISA.getServerConfiguration(options);
        // Merge data
        Ext.apply(options, optionsServer);
        WISA.data.GroupingClientDataset.superclass.constructor.call(this, options);
    }
});

/**
 * @class WISA.controls.Grid
 * @information The basic grid for all WISA applications.
 * @extends Ext.grid.GridPanel
 * @author Michael Van Canneyt
 * @version 0.9
 */
WISA.controls.Grid = Ext.extend(Ext.grid.GridPanel, {});

/**
 * @class WISA.controls.Dialog
 * @information The basic dialog for all WISA applications.
 * @extends Ext.Window
 * @author Michael Van Canneyt
 * @version 0.9
 */
WISA.controls.Dialog = Ext.extend(Ext.Window, {
    initComponent : function() {
        var options = this.BuildForm();
        if (!Ext.isEmpty(options)) {
            Ext.apply(this, options);
        }
        WISA.controls.Dialog.superclass.initComponent.call(this);
    },
    BuildForm : function() {
        return null;
    },
    ApplyParams : function(params) {
        // Apply any parameters you see fit
    }
});

/**
 * @class WISA.controls.Form
 * @information The basic form for all WISA applications.
 * @extends Ext.Panel
 * @author Michael Van Canneyt
 * @version 0.9
 */
WISA.controls.Form = Ext.extend(Ext.Panel, {
    /**
     * @method BuildForm
     * @information The BuildForm method creates the actual GUI. This method must be overridden and must build the FORM
     *              gui. It cannot perform any server calls.
     * @author Michael Van Canneyt
     */
    getFormTitle : function() {
        return 'Onbekend scherm';
    },
    initComponent : function() {
        var options = this.BuildForm();
        if (!Ext.isEmpty(options)) {
            Ext.apply(this, options);
        }
        WISA.controls.Form.superclass.initComponent.call(this);
    },
    BuildForm : function() {
        return null;
    },
    ApplyParams : function(params) {
        // Apply any parameters you see fit
    }
});

// /* Application */
/**
 * @class WISA.Application
 * @information De WISA Application class: er wordt slechts 1 instance van deze class gemaakt.
 * @extends Ext.util.Observable
 * @author Michael Van Canneyt
 * @version 0.9
 */
WISA.Application = Ext.extend(Ext.util.Observable, (function() {
    // /*** Private Static Properties [ var = ] ***/
    // /*** Private Static Methods [ var = ] ***/
    return {
        constructor : function(options) {
            // /*** Below has Access to All Properties/Methods ***/
            // /*** Public Properties [ this.prop = ] ***/
            // none

            // /*** Private Properties [ var prop = ] ***/
            var isAPILoaded = false;
            var isCGI = false;
            var isThemeComboVisible = false;
            var isMainmenuVisible = true;
            var isSubmenuVisible = true;
            var urlBase = '.';
            var urlApplication = '';
            var userId = -1;
            var userName = '';
            var userLastlogin = '(onbekend)';
            var menu = null;
            var submenuPanel = null;
            var submenuConfig = {
                region : 'west',
                id : 'wisa-submenu', // DEVart added id
		cls: 'wisa-menu',
                collapsible : true,
                collapsed : false, // DEVart changed true to false
                layout : 'fit',
                title : 'Menu',
                margins : '50 20 3 20', // DEVart changed top margin from 0 to 50
		cmargins: '0 3 3 3',
                width : 200
            };
            var FMainMenuConfig = null;
            var bottomPanelConfig = null;
            var formMain = 'motd';
            var formParent = null;
            var dialogActive = null;
            var sessionConfig = null;

            // /*** Public Methods [ this.fct = ] ***/ // & //
            // /*** Private Methods [ var fct = ] ***/
            // // Setters and Getters
            var setIsAPILoaded = function(value) {
                isAPILoaded = value;
            };
            var getIsAPILoaded = function() {
                return isAPILoaded;
            };
            var setIsCGI = function(value) {
                isCGI = value;
            };
            var getIsCGI = function() {
                return isCGI;
            };
            var setIsThemeComboVisible = function(value) {
                isThemeComboVisible = value;
            };
            var getIsThemeComboVisible = function() {
                return isThemeComboVisible;
            };
            var setIsMainmenuVisible = function(value) {
                isMainmenuVisible = value;
            };
            var getIsMainmenuVisible = function() {
                return isMainmenuVisible;
            };
            var setMainMenuConfig = function (value) { 
                FMainMenuConfig = Ext.apply(value,{
                    region: 'north', 
                    collapsible: true
                }); 
            };
            this.getMainMenuConfig = function () { 
                return FMainMenuConfig;
            };
            var setIsSubmenuVisible = function(value) {
                isSubmenuVisible = value;
            };
            var getIsSubmenuVisible = function() {
                return isSubmenuVisible;
            };
            var setUrlBase = function(value) {
                urlBase = value;
            };
            var getUrlBase = function() {
                return urlBase;
            };
            var setUrlApplication = function(value) {
                urlApplication = value;
            };
            this.getUrlApplication = function() {
                if (urlApplication !== '') {
                    return urlApplication;
                } else {
                    if (getIsCGI()) {
                        urlApplication = getUrlBase() + '/server.cgi';
                    } else {
                        urlApplication = getUrlBase() + '/server';
                    }
                    return urlApplication;
                }
            };
            this.getUrlAPI = function() {
                return (this.getUrlApplication() + '\/RPC\/API\/');
            };
            this.getUrlDialogs = function() {
                return (this.getUrlApplication() + '\/' + 'Forms\/');
            };
            this.getUrlForms = function() {
                return (this.getUrlApplication() + '\/' + 'Forms\/');
            };
            var setUserData = function(data) {
                userId = data.userid;
                userName = data.name;
                userLastlogin = data.lastlogin;
            };
            var setSessionConfig = function(data) {
                sessionConfig = data;
            };
            this.getUserId = function() {
                return userId;
            };
            this.getSessionConfig = function() {
                return sessionConfig;
            };
            this.getUserName = function() {
                return userName;
            };
            this.getUserLastLogin = function() {
                return userLastlogin;
            };
            var setMenu = function(value) {
                menu = value;
            };
            this.getMenu = function() {
                return menu;
            };
            var setSubmenuPanel = function(value) {
                submenuPanel = value;
            };
            this.getSubmenuPanel = function() {
                return submenuPanel;
            };
            var setSubmenuConfig = function(options) {
                submenuConfig = Ext.apply(options, {
                    region : 'west',
                    collapsible : true
                });
            };
            var getSubmenuConfig = function() {
                return submenuConfig;
            };
            var setBottomPanelConfig = function (value) { 
                 bottomPanelConfig = Ext.apply(AValue,{
                     region: 'south', 
                     collapsible: false
                 });
            };
            this.getBottomPanelConfig = function () { 
                return bottomPanelConfig;
            };
            var setFormMain = function(value) {
                formMain = value;
            };
            this.getFormMain = function() {
                return formMain;
            };
            var setFormParent = function(value) {
                formParent = value;
            };
            var getFormParent = function() {
                return formParent;
            };
            var setDialogActive = function(object) {
                dialogActive = object;
            };
            var getDialogActive = function() {
                return dialogActive;
            };
            // // API
            var checkIsAPILoaded = function() {
                if (true !== getIsAPILoaded()) {
                    return false;
                } else {
                    return true;
                }
            };
            var setLoadAPISuccess = function(response, options) {
                try {
                    eval(response.responseText);
                } catch (err) {
                    WISA.showAjaxError('AJAX error during API Load: ' + err.message);
                }
                Ext.Direct.addProvider(WISA.API);
                setIsAPILoaded(true);
                this.fireEvent('APILoaded');
            };
            var setLoadAPIFailure = function() {
                window.alert('WISA API could not be loaded');
            };
            this.loadAPI = function() {
                Ext.Ajax.request({
                    url : this.getUrlAPI(),
                    success : setLoadAPISuccess,
                    failure : setLoadAPIFailure,
                    scope : this
                });
            };
            this.buildMenu = function() {
                var o = [];
                var mnu = this.getMenu();
                if (mnu) {
                    for ( var m = 0; m < mnu.Items.length; m += 1) {
                        var mi = mnu.Items[m];
                        var extraCls = '';
                        if (m==0) {
                            extraCls += ' wisa-top-menu-item-first';
                        }
                        if (m==(mnu.Items.length-1)) {
                            extraCls += ' wisa-top-menu-item-last';
                        }
                        
                        o[m] = {
                            id : 'wisa-mainmenu-' + mi.Form, // DEVart added id
                            cls: 'wisa-top-menu-item' + extraCls, // DEVart added cls
                            xtype : 'button',
                            text : mi.Caption,
                            iconCls : mi.Icon,
                            tooltip : mi.Hint,
                            height : 92, // DEVart added height
                            handler : (function(mi) {
                                var it = mi;
                                return function(e) {
                                    if (!Ext.isEmpty(it.Items)) {
                                        this.showSubmenu(it);
                                    }
                                    this.showForm({
                                        form : it.Form
                                    });
                                };
                            }(mi)),
                            scope : this
                        };
                    }
                }
                o[o.length] = '->';
                if (getIsThemeComboVisible()) {
                    o[o.length] = {
                        xtype : 'themecombo'
                    };
                }
                o[o.length] = {
                    xtype : 'button',
                    text : this.getUserName(),
                    cls : 'wisa-profile',
                    iconCls : 'profile',
                    tooltip : 'Mijn gebruikersgegevens beheren',
                    handler : function() {
                        this.showForm({
                            form : 'myprofile'
                        });
                    },
                    scope : this
                };
                o[o.length] = {
                    xtype : 'button',
                    text : 'Afmelden',
                    cls : 'wisa-logout',
                    iconCls : 'logout',
                    tooltip : 'Afmelden en terug naar het aanmeldscherm',
                    handler : function() {
                        this.logout();
                    },
                    scope : this
                };
                return o;
            };
            this.createMainForm = function() {
                var it = [];
                Ext.QuickTips.init();
                setFormParent(new Ext.Panel({
                    title : 'A parent',
                    border : true,
                    region : 'center',
                    id : 'wisa-center', // DEVart added id
		    cls: 'wisa-center',
                    margins : '50 3 3 0', // DEVart changed margin top from 0 to 50
		    cmargins: '0 3 3 0',
                    layout : 'fit'
                }));
                if (getIsSubmenuVisible()) {
                    setSubmenuPanel(new Ext.Panel(getSubmenuConfig()));
                    it[it.length] = this.getSubmenuPanel();
                }
                if (getIsMainmenuVisible()) {
                    var t = this.getMainMenuConfig();
                    if (Ext.isEmpty(t)) {
                        t = {
                            xtype : 'toolbar',
                            region : 'north',
			    cls: 'wisa-top-toolbar',
			    margins: '0 0 0 0',
                            padding : 0,
                            border : false,
                            height : 89, // DEVart changed height from 28 to 89
                            items : this.buildMenu()
                        };
                    };
                    it[it.length] = t;
                };
                var c = this.getBottomPanelConfig();
                if (! Ext.isEmpty(c)) {
                  it[it.length]=c;
                }
                it[it.length] = getFormParent();
                this.ViewPort = new Ext.Viewport({
                    layout : 'border',
                    items : it
                });
            };
            this.showSubmenu = function(menu) {
                var p = this.getSubmenuPanel();
                p.setTitle(menu.Caption);
                p.removeAll(true);
                var r = new Ext.tree.TreeNode({
                    text : menu.Caption,
                    expanded : true
                });
                for ( var m = 0; m < menu.Items.length; m += 1) {
                    var mi = menu.Items[m];
                    var o = {
                        text : mi.Caption,
                        iconCls : mi.Icon,
                        leaf : true,
                        qtip : mi.Hint,
                        listeners : {
                            'click' : {
                                fn : (function(mi) {
                                    var it = mi;
                                    return function(e) {
                                        this.showForm({
                                            form : it.Form
                                        });
                                    };
                                }(mi)),
                                scope : this
                            }
                        }
                    };
                    r.appendChild(o);
                }
                var t = new Ext.tree.TreePanel({
                    root : r
                });
                p.add(t);
                if (p.collapsed) {
                    p.toggleCollapse();
                }
                p.doLayout();
            };
            // // Dialogs and Forms
            this.showDialog = function(options) {
                if ((Ext.isEmpty(WISA.Forms[options.form])) && ('noloop' !== arguments[arguments.length - 1])) {
                    /* */console.log('Loading dialog ' + options.form + '.js');
                    options = Ext.apply(options, {
                        url : this.getUrlDialogs() + options.form + '.js',
                        success : setLoadDialogSuccess,
                        failure : setLoadDialogFailure,
                        scope : this
                    });
                    Ext.Ajax.request(options);
                } else {
                    var objForm = new WISA.Forms[options.form];
                    setDialogActive(objForm);
                    objForm.show();
                    objForm.ApplyParams(options.Params);
                    objForm.focus();
                }
            };
            var setLoadDialogSuccess = function(response, options) {
                try {
                    eval(response.responseText);
                } catch (err) {
                    WISA.showAjaxError('AJAX error : ' + err.message);
                }
                this.fireEvent('DialogLoaded', options.form); // TODO Check Change: dialog dependant
                if (Ext.isEmpty(WISA.Forms[options.form])) {
                    WISA.showAjaxError('"' + options.form + '" did not define a valid dialog');
                } else {
                    if (!Ext.isFunction(WISA.Forms[options.form])) {
                        WISA.showAjaxError('"' + options.form + '" is not a valid form constructor');
                    } else {
                        this.showDialog(options, 'noloop');
                    }
                }
            };
            var setLoadDialogFailure = function(response, options) {
                window.alert('Kan scherm \"' + options.form + '\" niet openen: ' + response.statusText);
            };
            var closeDialogActive = function() {
                if (getDialogActive()) {
                    getDialogActive().close();
                }
            };
            this.showForm = function(options) {
                if (Ext.isEmpty(getFormParent())) {
                    this.createMainForm();
                }
                if ((Ext.isEmpty(WISA.Forms[options.form])) && ('noloop' !== arguments[arguments.length - 1])) {
                    /* */console.log('Loading form ' + options.form + '.js');
                    options = Ext.apply(options, {
                        url : this.getUrlForms() + options.form + '.js',
                        success : setLoadFormSuccess,
                        failure : setLoadFormFailure,
                        scope : this
                    });
                    Ext.Ajax.request(options);
                } else {
                    if ('noloop' == arguments[arguments.length - 1]) {
                        var clear = true;
                        var objForm = new WISA.Forms[options.form];
                        var objFormParent = getFormParent();
                        if (clear) {
                            objFormParent.removeAll(true);
                        }
                        objFormParent.add(objForm);
                        objFormParent.doLayout();
                        objForm.ApplyParams(options.Params);
                        objFormParent.setTitle(objForm.getFormTitle());
                        return false; // stop task.
                    } else { // MVC: Workaround voor IE resource/memory leak. Niet dadelijk nieuwe form tonen, maar in timed event.
                      Ext.TaskMgr.start({
                        run: this.showForm,
                        args: [options,'noloop'],
                        interval: 100,
                        scope: this,
                        repeat: 1
                      });
                    }
                }
            };
            var setLoadFormSuccess = function(response, options) {
                try {
                    eval(response.responseText);
                } catch (err) {
                    WISA.showAjaxError('AJAX error : ' + err.message);
                }
                this.fireEvent('FormLoaded', options.form); // TODO Check Change: form dependant
                if (Ext.isEmpty(WISA.Forms[options.form])) {
                    WISA.showAjaxError('"' + options.form + '" did not define a valid form');
                } else {
                    if (!Ext.isFunction(WISA.Forms[options.form])) {
                        WISA.showAjaxError('"' + options.form + '" is not a valid form constructor');
                    } else {
                        this.showForm(options, 'noloop');
                    }
                }
            };
            var setLoadFormFailure = function(response, options) {
                window.alert('Kan scherm \"' + options.form + '\" niet openen: ' + response.statusText);
            };
            // // User Management
            this.login = function(options) {
                WisaUserAPI.Authenticate(options.user, options.password, handleLogin.createDelegate(this));
            };

            this.showLogin = function(options) {
                if (checkIsAPILoaded()) {
                    this.showDialog(Ext.apply({
                        form : 'login'
                    }, options));
                } else {
                    /* */console.log('Waiting for API...');
                    this.addListener('APILoaded', function() {
                        this.showLogin(options);
                    }, this, {
                        single : false
                    });
                }
            };
            var realLogout = function () {
                var strLogoutURL = getUrlBase() + '/start.html';
                WisaUserAPI.Logout(function() {
                    window.location = strLogoutURL;
                });
            };
            /* Auto-logout functionality */
            var autoRefreshProcId = 0;
            var autoLogout = function() {
              realLogout();
            };
            var startLogoutTimer = function() {
                if (autoRefreshProcId) {
                    clearInterval(autoRefreshProcId);
                }
                autoRefreshProcId = setInterval(autoLogout.createDelegate(this, []), (Application.getSessionConfig().SessionTimeOut - 1)*60000);
            };
            var stopLogoutTimer = function(){
               if(autoRefreshProcId){
                  clearInterval(autoRefreshProcId);
               }
            };
            var handleLogin = function(provider, response) {
                if (!Ext.isEmpty(response.error)) {
                    WISA.showAjaxError('Fout bij aanmelden: ' + response.error.message);
                } else {
                    if (!Ext.isEmpty(response.result.SessionConfig)) {
                      setSessionConfig(response.result.SessionConfig);
                      if (!Ext.isEmpty(this.getSessionConfig().SessionTimeOut)) {
                        startLogoutTimer();
                        Ext.Ajax.on('beforerequest', startLogoutTimer, this);
                      }
                    }
                    Ext.getCmp('wisa-login-logo').destroy(); // DEVart hide logo
                    setUserData(response.result.UserData);
                    setMenu(response.result.menu);
                    closeDialogActive();
                    if (this.hasListener('Login')) {
                        this.fireEvent('Login');
                    } else {
                        this.showForm({
                            form : this.getFormMain()
                        });
                    }
                }
            };
            this.logout = function() {
               Ext.MessageBox.confirm('Afmelden','Bent u zeker dat u de sessie wil afbreken ?',
                 function (bid) {  
                   if (bid=='yes') {
                     realLogout();
                   }
                }, 
                this
               );
            };
            // Compatibility
            this.getShowThemeCombo = this.getIsThemeComboVisible;
            this.setShowMainMenu = this.setIsMainmenuVisible;
            this.getShowMainMenu = this.getIsMainmenuVisible;
            this.setShowSubMenu = this.setIsSubmenuVisible;
            this.getShowSubMenu = this.getIsSubmenuVisible;
            this.getBaseURL = this.getUrlBase;
            this.getAppURL = this.getUrlApplication;
            this.saveUserData = this.setUserData;
            this.getUserID = this.getUserId;
            this.getLastLogin = this.getUserLastLogin;
            this.setSubMenuConfig = this.setSubmenuConfig;
            this.getSubMenuConfig = this.getSubmenuConfig;
            this.setMainForm = this.setFormMain;
            this.getMainForm = this.getFormMain;
            this.setActiveDialog = this.setDialogActive;
            this.getActiveDialog = this.getDialogActive;
            this.closeActiveDialog = this.closeDialogActive;
            this.ShowDialog = this.showDialog;
            this.ShowForm = this.showForm;
            this.ShowLogin = this.showLogin;
            this.HandleLogin = this.handleLogin;
            this.DoLogin = this.login;
            this.DoLogout = this.logout;
            this.CreateMainForm = this.createMainForm;
            this.BuildMenu = this.buildMenu;

            // Constructor Code
            // // Configuration
            if (Ext.isDefined(options.url)) {
              if (Ext.isDefined(options.url.base)) {
                  setUrlBase(options.url.base);
              }
              if (Ext.isDefined(options.url.base)) {
                  setUrlBase(options.url.base);
              }
              if (Ext.isDefined(options.url.app)) {
                  setUrlApplication(options.url.app);
              }
            }
            if (Ext.isDefined(options.hasThemes)) {
                setIsThemeComboVisible(options.hasThemes);
            }
            if (Ext.isDefined(options.mainmenu)) {
              if (Ext.isDefined(options.mainmenu.visible)) {
                setIsMainmenuVisible(options.mainmenu.visible);
              }
              if (Ext.isDefined(options.mainmenu.config)) {
                setMainMenuConfig(options.mainmenu.config);
              }
            }
            if (Ext.isDefined(options.submenu)) {
              if (Ext.isDefined(options.submenu.visible)) {
                  setIsSubmenuVisible(options.submenu.visible);
              }
              if (Ext.isDefined(options.submenu.config)) {
                  setSubmenuConfig(options.submenu.config);
              }
            }
            if (Ext.isDefined(options.mainform)) {
              if (Ext.isDefined(options.mainform.name)) {
                  setFormMain(options.mainform.name);
              }
            } 

            // // Events
            this.addEvents({
                'APILoaded' : true,
                'FormLoaded' : true,
                'DialogLoaded' : true,
                'Login' : true,
                'Logout' : true
            });
            // // Superclass Constructor
            WISA.Application.superclass.constructor.call(this, options);
            // // Initialisation
            this.loadAPI();
        }
    // /*** Below has Access to All except Constructor ***/
    // /*** Public Properties [ prop : ] ***/
    // /*** Public Methods [ fct : ] ***/
    };
}()));

WISA.controls.SingletonApplication = Ext.extend(Ext.util.Observable, (function() {
    // Private Hidden Property - stays alive as long as reference exists
    var objSingletonInstance = null;
    // Return Object
    return {
        constructor : function(options) {
            if (!objSingletonInstance) {
                objSingletonInstance = new WISA.Application(options);
            }
            return objSingletonInstance;
        }
    };
}()));

/* functie bij Date om het juiste formaat voor een datum parameter te krijgen in een string */
Date.prototype.asWisaParameter = function () {
  return this.format('d/m/Y');
}


// /* Backwards Compatibility & Future Changes */ 
WISA.ShowError = WISA.showError;
WISA.AjaxError = WISA.showAjaxError;
WISA.CheckError = WISA.getResponseError;
WISA.CreateServerConfig = WISA.getServerConfiguration;

WISA.ClientDataset = WISA.data.ClientDataset;
WISA.GroupingClientDataset = WISA.data.GroupingClientDataset;
WISA.Dialog = WISA.controls.Dialog;
WISA.Grid = WISA.controls.Grid;
WISA.Form = WISA.controls.Form;
WISA.FormLoader = WISA.controls.FormLoader;

/* baseforms */

/***************
 CustomGridForm: basisform die een grid toont met knoppen toevoegen, bewerken verwijderen
****************/
WISA.baseforms.CustomGridForm = Ext.extend(WISA.Form, {
  /* velden met gegevens en componenten */
  pFilter: null,
  pOverzicht: null,
  itemList: null,
  dsOverzicht: null,
  selOverzicht: null,
  positionID: -1,

  /* functions to override in descendants to configure the form */

  buildFilterConfig: function () {
    return {};
  },

  buildGridColumns: function () {
    return [
      {header: 'Titel', dataIndex: 'VELDNAAM'}
      ]
  },

  getProviderName: function () {
    return 'ProviderToUse';
  },

  getIDField: function () {
    return 'prefix_ID';
  },

  getGridIsPaging: function () {
    return false;
  },

  buildExtraButtons: function () {
    return [];
  },

  getFicheClass: function () {
    /* descendants moeten de naam van een descendant van CustomFicheDialog teruggeven */
    return 'CustomFicheDialog';
  },

  /* functies voor interene werking */

  doeReload: function (data) {
    var opt = this.dsOverzicht.lastOptions;
    this.positionID = data.ID;
    Ext.apply(opt, {
      callback: this.doePosition,
      scope: this
    });
    this.dsOverzicht.reload(opt);
  },

  doePosition: function() {
    this.selOverzicht.selectRecords([this.dsOverzicht.getById(this.positionID)]);
  },

  doeToevoegen: function () {
    this.editRecord (-1);
  },

  doeBewerken: function () {
    this.editRecord (this.getSelectedID());
  },

  doeVerwijderen: function () {
  },

  editRecord: function (anID) {
    var fiche = this.getFicheClass();
    var idfield = this.getIDField();
    var params = {
      ID: anID,
      OKHandler: this.doeReload,
      OKScope: this
      };
    params[idfield] = anID;
    Application.ShowDialog({
      form: fiche,
      Params: params
    });
  },

  getSelectedID: function () {
    var rec = this.getSelected();
    if (Ext.isEmpty(rec)) {
      return -1;
    } else {
      return rec.get(this.getIDField());
    }
  },

  getSelected: function () {
    if (Ext.isEmpty(this.selOverzicht)) {
      return null;
    } else {
      return this.selOverzicht.getSelected();
    }
  },
  getGridTitle : function () {
    return this.getFormTitle();
  },

  /* Build the actual form contents*/

  BuildForm: function () {
    var config = this.buildFilterConfig();
    var gridcolumns = this.buildGridColumns();
    var buttonconfig = this.buildExtraButtons();
    var gridconfig = null;
    var itemList = [];

    if (!Ext.isEmpty(config)) {
      this.pFilter = new WISA.controls.Panel(config);
      itemList.push(this.pFilter);
    };
    this.dsOverzicht = new WISA.data.ClientDataset({
      ProviderName: this.getProviderName(),
      autoSave: false,
      autoLoad: false
    });
    this.selOverzicht = new Ext.grid.RowSelectionModel({
      singleSelect: true
    });
    buttonconfig.push({
        text: 'Toevoegen',
        handler: this.doeToevoegen,
        scope: this
        });
    buttonconfig.push({
        text: 'Bewerken',
        handler: this.doeBewerken,
        scope: this
        });
    buttonconfig.push({
        text: 'Verwijderen',
        handler: this.doeVerwijderen,
        scope: this
        });
    gridconfig = {
      border: true,
      title: this.getGridTitle(),
      height: 400,
      width: 600,
      store: this.dsOverzicht,
      selModel: this.selOverzicht,
      columns: gridcolumns,
      tbar: buttonconfig,
    };
    if (this.getGridIsPaging()) {
      gridconfig.start=0;
      gridconfig.limit=100;
    };
    this.pOverzicht = new WISA.controls.Grid(gridconfig);
    itemList.push(this.pOverzicht);
    return {
      layout: 'vbox',
      layoutConfig: {align: 'stretch', pack: 'center', defaultMargins: {top:20, right:50, bottom:20, left:50}},
      items: itemList,
      };
  }
});

/******************
 CustomFicheDialog: Dialoog basis opgeroepen door overzichtsform
*******************/

WISA.baseforms.CustomFicheDialog = Ext.extend(WISA.Dialog, {
  /* interne velden */
  onSave: null,
  recordID: -1,
  dsFiche: null,
  fFicheForm : null,
  theRecord: null,

  /* function to override in descendant to configure the form */

  getProviderName: function () {
    return 'ProviderToUse';
  },

  getIDField: function () {
    return 'prefix_ID';
  },

  buildExtraButtons: function () {
    return []; // configuration array met extra buttons
  },

  loadRecord : function (rec) {  // Called when the data is loaded, rec is het te tonen datarecord
    this.fFicheForm.loadRecord(this.theRecord);
    if (Ext.isDefined(this.doLoadRecord)) {
      this.doLoadRecord(rec);
    }
  },

  updateRecord: function (rec) {  // Called when the Bewaren button is pressed, rec is het record van de dataset
    this.fFicheForm.updateRecord(this.theRecord);
    if (Ext.isDefined(this.doUpdateRecord)) {
      rec.beginEdit();
      this.doUpdateRecord(rec);
      rec.endEdit();
    }
  },

  buildFiche: function () {  // dsFiche zal reeds gecreeerd zijn als deze methode wordt opgeroepen
    return {}; // configuration object for a panel that shows the fiche
  },

  /* functies voor interne werking */

  afterSave: function () {
    this.fireEvent('saveOK', {ID: this.dsFiche.getAt(0).get(this.getIDField())});
    this.close();
  },

  afterLoad: function () {
    if (this.dsFiche.getCount() === 0) {
      this.theRecord = new this.dsFiche.recordType();
    } else
      this.theRecord = this.dsFiche.getAt(0);
    this.loadRecord(this.theRecord);
  },

  onException: function (proxy, type, action, options, res, arg) {
    if (type === 'remote') {
      WISA.showError({title: 'Fout met verwerken data', msg: res.message});
    } else {
      WISA.showError({title: 'Fout met data', msg: 'Fout bij ophalen/verwerken van data '});
    }
  },

  doeBewaren: function () {
    var rec = this.theRecord;
    this.updateRecord(rec);
    if (this.dsFiche.getCount() === 0) {
      this.dsFiche.add(this.theRecord);
    } else {
      this.dsFiche.save();
    }
  },

  doeAnnuleren: function () {
    this.close();
  },

  /* Build the actual form contents*/

  BuildForm: function () {
    var ficheCfg = this.buildFiche();
    var buttonconfig = this.buildExtraButtons();
    this.dsFiche = new WISA.data.ClientDataset({
        ProviderName: this.getProviderName(),
        autoLoad: false,
        listeners: {
          "save": this.afterSave,
          "load": this.afterLoad,
          'exception' : this.onException,
          scope: this
        }
      });
    // verplichte buttons toevoegen
    buttonconfig.push({
      text: 'Bewaren',
      handler: this.doeBewaren,
      scope: this
    });
    buttonconfig.push({
      text: 'Annuleren',
      handler: this.doeAnnuleren,
      scope: this
    });
    Ext.apply(ficheCfg,{buttons: buttonconfig});
    // Form object maken.
    this.fFicheForm = new Ext.form.FormPanel(ficheCfg);
    return { items : [this.fFicheForm], defaultButton: ficheCfg.items[0] }
  },

  /* Process parameters which have been passed here */
  ApplyParams : function (Params) { // Zorg dat deze methode altijd uitgevoerd wordt !!
    if ( Params != undefined ) {
      /* Call inherited ApplyParams */
      WISA.baseforms.CustomFicheDialog.superclass.ApplyParams.call(this, Params);
      if ( Params.OKHandler != undefined ) {
        this.addListener("saveOK",Params.OKHandler,Params.OKScope);
      this.recordID = Params.ID;
      }
    }
  }

});

